@@ -2,13 +2,14 @@
 #define INIT_H_
 
 /* use the fastest baud-rate */
-#if defined(B4000000)
-# define BAUDRATE	B4000000
-#elif defined(B921600)
-# define BAUDRATE	B921600
-#elif defined(B115200)
-# define BAUDRATE	B115200
-#elif defined(B38400)
+//#if defined(B4000000)
+//# define BAUDRATE	B4000000
+//#elif defined(B921600)
+//# define BAUDRATE	B921600
+//#elif defined(B115200)
+//# define BAUDRATE	B115200
+//#elif defined(B38400)
+#if defined(B38400)
 # define BAUDRATE	B38400
 #elif defined(B19200)
 # define BAUDRATE	B19200
