@@ -4,7 +4,7 @@
  *
  * All portions of code are copyright by their respective author/s.
  * Copyright (c) 2005      WU Fengguang
- * Copyright (c) 2005-2006 Marc Lehmann <pcg@goof.com>
+ * Copyright (c) 2005-2006 Marc Lehmann <schmorp@schmorp.de>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -28,7 +28,6 @@
 
 #include <inttypes.h>
 
-#include "feature.h"
 #include "rxvtutil.h"
 
 #define KEYSYM_HASH_BITS        4       /* lowest #bits of keysym is used as hash key */
@@ -47,17 +46,11 @@
 #endif
 
 struct rxvt_term;
-struct keysym_t;
-
-typedef void (keyevent_handler) (rxvt_term *rt,
-                                 keysym_t *key,
-                                 KeySym keysym,
-                                 unsigned int state);
 
 struct keysym_t
 {
   enum keysym_type {
-    STRING, LIST, BUILTIN,
+    STRING, BUILTIN,
   };
 
   KeySym      keysym;
@@ -65,7 +58,6 @@ struct keysym_t
   /* the higher bits are preserved for Meta/NumLock keys */
   /* which are mapped to corresponding lower bits at register time */
   uint16_t    state;    /* indicates each modifiers' DOWN/UP status         */
-  uint16_t    range;    /* =1: single keysym; >1: a of range keysyms        */
   keysym_type type;
   const char  *str;      /* would normally be a keycode translation in UTF-8 */
 };
@@ -78,6 +70,7 @@ public:
 
   void clear ();
   void register_user_translation (KeySym keysym, unsigned int state, const char *trans);
+  void register_translation (KeySym keysym, unsigned int state, char *translation);
   void register_done ();        // call this to make newly registered keymaps take effect
   bool dispatch (rxvt_term *term, KeySym keysym, unsigned int state);
 
